"use strict";

var logger = require('../../logger');

var ObjectWithListeners = require('../../ObjectWithListeners');

/**
 * Class representing abstract overlay element on the map relevant for a
 * specific layout.
 */
function AbstractOverlayElement(params) {
  var self = this;
  // call super constructor
  ObjectWithListeners.call(this);

  self.registerListenerType("onClick");
  self.setCustomMap(params.map);
  if (params.element !== undefined && params.element !== null) {
    self.setIdentifiedElement(params.element);
  }
  self._googleMapObjects = [];

  if (params.onClick !== undefined) {
    if (typeof params.onClick !== "function") {
      for (var i = 0; i < params.onClick.length; i++) {
        self.addListener("onClick", params.onClick[i]);
      }
    } else {
      self.addListener("onClick", params.onClick);
    }
  }
}

AbstractOverlayElement.prototype = Object.create(ObjectWithListeners.prototype);
AbstractOverlayElement.prototype.constructor = AbstractOverlayElement;

AbstractOverlayElement.prototype.setMap = function(map) {
  for (var i = 0; i < this.getGoogleMapObjects().length; i++) {
    this.getGoogleMapObjects()[i].setMap(map);
  }
};

AbstractOverlayElement.prototype.getBounds = function() {
  var bounds = new google.maps.LatLngBounds();
  for (var i = 0; i < this.getGoogleMapObjects().length; i++) {
    bounds.extend(this.getGoogleMapObjects()[i].getBounds().getSouthWest());
    bounds.extend(this.getGoogleMapObjects()[i].getBounds().getNorthEast());
  }
  return bounds;
};

AbstractOverlayElement.prototype.isShown = function() {
  var result = false;
  for (var i = 0; i < this.getGoogleMapObjects().length; i++) {
    var map = this.getGoogleMapObjects()[i].getMap();
    if (map !== null && map !== undefined) {
      result = true;
    }
  }

  return result;
};

AbstractOverlayElement.prototype.show = function() {
  if (this.isShown()) {
    logger.warn("Surface already shown");
    return;
  }
  this.setMap(this.getCustomMap().getGoogleMap());
};

AbstractOverlayElement.prototype.hide = function() {
  this.setMap(null);
};

AbstractOverlayElement.prototype.onClickHandler = function() {
  return this.callListeners("onClick");
};

AbstractOverlayElement.prototype.getGoogleMarker = function() {
  return this.getGoogleMapObjects()[0];
};

AbstractOverlayElement.prototype.getGoogleMapObjects = function() {
  return this._googleMapObjects;
};

AbstractOverlayElement.prototype.addGoogleMapObject = function(googleObject) {
  this._googleMapObjects.push(googleObject);

  var self = this;
  var onclick = function() {
    return self.onClickHandler();
  };
  google.maps.event.addListener(googleObject, 'click', onclick);
};

AbstractOverlayElement.prototype.getIdentifiedElement = function() {
  return this._identifiedElement;
};

AbstractOverlayElement.prototype.getModelId = function() {
  return this.getIdentifiedElement().getModelId();
};

AbstractOverlayElement.prototype.updateIdentifiedElement = function(identifiedElement) {
  if (identifiedElement.getColor() !== undefined) {
    this.setColor(identifiedElement.getColor());
  }
};

AbstractOverlayElement.prototype.setIdentifiedElement = function(identifiedElement) {
  this._identifiedElement = identifiedElement;
};

AbstractOverlayElement.prototype.getBounds = function() {
  var self = this;
  var bounds = new google.maps.LatLngBounds();
  for (var i = 0; i < self.getGoogleMapObjects().length; i++) {
    var elementBounds = self.getGoogleMapObjects()[i].getBounds();
    bounds.extend(elementBounds.getNorthEast());
    bounds.extend(elementBounds.getSouthWest());
  }

  return bounds;
};

/**
 * Returns {@link AbstractCustomMap} where surface is located.
 * 
 * @returns {@link AbstractCustomMap} where surface is located
 */
AbstractOverlayElement.prototype.getCustomMap = function() {
  return this._customMap;
};

AbstractOverlayElement.prototype.setCustomMap = function(customMap) {
  this._customMap = customMap;
};

module.exports = AbstractOverlayElement;
